import 'package:bson/bson.dart';
import 'package:meta/meta.dart';

import '../proto.dart' as p;

/// A result type returned by an InsertMany operation.
///
/// ## Reference
/// See https://pkg.go.dev/go.mongodb.org/mongo-driver@v1.10.3/mongo#InsertManyResult.
class InsertManyResult {
  /// The _id values of the inserted documents.
  /// Values generated by the driver will be of type primitive.ObjectID.
  final List<dynamic> insertedIds;
  InsertManyResult({
    required this.insertedIds,
  });

  @internal
  factory InsertManyResult.fromProto(p.InsertManyResult pResult) {
    final ids = pResult.insertedIds.map(
      (e) {
        final bson = BSON();
        final bb = BsonBinary.from(e);
        final idDoc = bson.deserialize(bb);
        final id = idDoc['_id'];
        return id;
      },
    ).toList();

    final ret = InsertManyResult(insertedIds: ids);
    return ret;
  }
}
